function desc = GFD(im, nRad, nAng)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% creator: Santosh 
% input: im = binary_image, {nRad = 4, nAng = 9 according to the original paper,
% But if you want to tune the parameters do the following: 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for nRad = 4:9    
%     for nAng = 9:18 
% 	desc = GFD(im, nRad, nAng);
%     end
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% 1. Test, Input Image
% nRad = 4; nAng = 9;
%% 2. centroid
[xobj, yobj] = find(im == 1);
xc = ceil(mean(xobj));
yc = ceil(mean(yobj));
% disp(['centroid : ' num2str(xc) ' x ' num2str(yc)]);

%% 3. set the centroid as the origin and get macimum radius of the shape (maxRad)
d = sqrt((xobj-xc).^2 + (yobj-yc).^2);
maxRad = round(max(d(:)));
% disp(['Radius max : ' num2str(maxRad)]);

%% 4. Polar Fourier transform
FR = zeros(nRad,nAng);
FI = zeros(nRad,nAng);
for rad=1:nRad
    for ang=1:nAng
        for x=1:size(im,1)
            for y=1:size(im,2)
                radius = sqrt((x - xc).^2 + (y - yc).^2);
                theta = atan2((y - yc),(x - xc)); % theta in[-pi, pi]
                if theta < 0; theta = theta + 2*pi; end; % theta in [0, 2pi]
                
                %V1 : phi =  2*pi*((rad-1)*radius/maxRad + (ang-1)*theta);
                %V2 : phi =  2*pi*(rad-1)*radius/maxRad + (ang-1)*theta;
                phi = 2*pi*(rad-1)*radius/maxRad + (ang-1)*theta;                
                FR(rad,ang) = FR(rad,ang) + im(x,y)*cos(phi); %Real part of spectra
                FI(rad,ang) = FI(rad,ang) - im(x,y)*sin(phi); %Imaginary part of spectra
                
            end
        end
    end
end

%% 5. Computation of Fourier Descriptors
FD = zeros(1,nRad *nAng);
for rad=1:nRad
    for ang=1:nAng
        if ((rad ==1) && (ang==1)); 
           DC =  sqrt( (FR(1,1)^2 + FI(1,1)^2) );
           FD(1) = DC / (pi*maxRad^2); 
        else 
           FD((rad-1)*nAng+ang) = sqrt(FR(rad,ang)^2+FI(rad,ang)^2)/DC; 
        end
    end
end

%% 6. return
desc = FD;

%% 7. plot  
% %% Display signature/descriptor plot
% plot(desc,'-rs','LineWidth',1,...
%                 'MarkerEdgeColor','k',...
%                 'MarkerFaceColor','g',...
%                 'MarkerSize',4)

end
