%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Santosh K.C.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dirName = 'ShapeContext';

cd(dirModelParts);
    
fileName = dir(fileType);

% shapeContextDist: distance matrix
shapeContextDist = zeros(length(fileName));

% imMatchIndx: indices of the best matched images
imMatchIndx = zeros(imageRelevantRank+1,length(fileName));

for j = 1:length(fileName)
    [pathstr, name] = fileparts(fileName(j).name);

    % load data saved by Zernike
    load(fullfile(dirName,strcat('contour_',name,'.mat')));

    % zernikeMomentsCur: the query image
    contourPointsCur = contourPoints;
    for i = 1:length(fileName)
        disp(['     ' dirName ...
            ', query image j = ',num2str(j),...
            ', comparing image i = ',num2str(i)]);
        [pathstr, name] = fileparts(fileName(i).name);

        % load data saved by Zernike
        load(fullfile(dirName,strcat('contour_',name,'.mat')));

        % measure the distance
        shapeDistance;
        shapeContextDist(i,j) = shapeDist;
    end

    % get the indices of the best matched images
    [shapeContextDistSorted,shapeContextDistSortedIndx] = ...
        sort(shapeContextDist(:,j));
    imMatchIndx(:,j) = ...
        shapeContextDistSortedIndx(1:(imageRelevantRank+1));
end
    
% convert the indices of the best matched images into binary
%   1 : if the index is in the same class with the query
%   0 : otherwise
imMatchBinary = zeros(size(imMatchIndx));
for j = 1:length(fileName)
    for i = 1:(imageRelevantRank+1)
        class = floor((j-1)/imagePerClass) + 1;
        if (imMatchIndx(i,j) > imagePerClass*(class-1)) &&...
                (imMatchIndx(i,j) < (imagePerClass*class+1))
            imMatchBinary(i,j) = 1;
        end
    end
end

% precision & recall
imMatchPrecision = cumsum(sum(imMatchBinary,2)/length(fileName))...
    ./(1:imagePerClass)';
imMatchRecall = cumsum(sum(imMatchBinary,2))...
    /(length(fileName)*imagePerClass);

% recall for each class
classNum = length(fileName)/imagePerClass;
imMatchClass = zeros(imageRelevantRank+1,classNum);
for j = 1:classNum
    imMatchClass(:,j) = sum(imMatchBinary(:,...
        ((j-1)*imagePerClass+1):j*imagePerClass),2);
end
imMatchClass';

% precision of the retrieval
imMatchPercent = sum(imMatchBinary,2)/length(fileName)*100;

clf, plot(imMatchRecall,imMatchPrecision,...
    '-s','LineWidth',2,'MarkerSize',10);
axis([0 1 0 1.1]); grid on;
title(strcat('match_PreRecall_',dirName,'.png'),'interpreter','none');
saveas(1,fullfile(pathCur,dirDatabaseData,...
    strcat('match_PreRecall_',dirName,'.png')));

clf, plot(imMatchPercent(1:(imageRelevantRank+1)),...
    '-x','LineWidth',2,'MarkerSize',10);
axis([1 (imageRelevantRank+1) 0 100]); grid on;
title(strcat('match_Rank_',dirName,'.png'),'interpreter','none');
saveas(1,fullfile(pathCur,dirDatabaseData,...
    strcat('match_Rank_',dirName,'.png')));

save(fullfile(pathCur,dirDatabaseData,...
    strcat('match_',dirName,'.mat')),...
    'shapeContextDist','imMatchClass','imMatchIndx','imMatchBinary',...
    'imMatchPercent','imMatchPrecision','imMatchRecall');

cd(pathCur);
